@@ -22,8 +22,8 @@ else:
 
 # add local dir to search path
 sys.path.insert(0,os.path.dirname(sys.path[0]))
-from cdo import Cdo,CDOException,CdoTempfileStore
-
+from cdo import Cdo, CDOException
+import cdo as cdoPkg
 
 if 'CDF_MOD' in os.environ:
   CDF_MOD = os.environ['CDF_MOD']
@@ -40,6 +40,7 @@ MAINTAINERMODE = 'MAINTAINERMODE' in os.environ
 
 if SHOW:
   from matplotlib import pylab as pl
+  from matplotlib import pyplot as plt
 
 def plot(ary,ofile=False,title=None):
   if not SHOW:
@@ -84,9 +85,9 @@ class CdoTest(testClass):
         print(cdo.sinfov(input='-topo'))
 
     def testCDO(self):
+        self.assertTrue(parse_version(cdoPkg.__version__) >= parse_version('1.6.0'))
         cdo = Cdo()
         print('this is CDO version %s'%(cdo.version()))
-        print('cdo-bindings version: %s'%(cdo.__version__()))
         newCDO="/home/ram/src/tools/spack/opt/spack/linux-antergos-skylake/gcc-11.1.0/cdo-1.9.9-switof25xqmlys765t7aonnc564wl3jl/bin/cdo"
         if os.path.isfile(newCDO):
             cdo.setCdo(newCDO)
@@ -185,8 +186,14 @@ class CdoTest(testClass):
           '2001-01-01T19:00:00',
           '2001-01-01T20:00:00',
           '2001-01-01T21:00:00']
-        self.assertEqual(timesExpected,
-                         cdo.showtimestamp(input="-settaxis,2001-01-01,12:00,1hour -for,1,10", autoSplit='  '))
+        if (parse_version(cdo.version()) == parse_version('2.0.0')):
+          # cdo-2.0.0 does not put output on a single line with '-s' like previous and later versions'
+          timesExpected = [['2001-01-01T12:00:00', '2001-01-01T13:00:00', '2001-01-01T14:00:00', '2001-01-01T15:00:00'],
+                           ['2001-01-01T16:00:00', '2001-01-01T17:00:00', '2001-01-01T18:00:00', '2001-01-01T19:00:00'],
+                           ['2001-01-01T20:00:00', '2001-01-01T21:00:00']]
+
+        actualOutput = cdo.showtimestamp(input="-settaxis,2001-01-01,12:00,1hour -for,1,10", autoSplit='  ')
+        self.assertEqual(timesExpected, actualOutput)
 
         self.assertEqual(['P T'],cdo.showname(input="-stdatm,0"))
         self.assertEqual(['P','T'],cdo.showname(input="-stdatm,0",autoSplit=' '))
@@ -207,8 +214,7 @@ class CdoTest(testClass):
 
     def test_chain(self):
         cdo = Cdo()
-        ofile = cdo.setname("veloc", input=" -copy -random,r1x1",options = "-f nc")
-        self.assertEqual(["veloc"],cdo.showname(input = ofile))
+        self.assertEqual(["veloc"],cdo.showname(input = cdo.setname("veloc", input=" -copy -random,r1x1",options = "-f nc")))
 
     def test_pychain(self):
         cdo = Cdo()
@@ -221,15 +227,20 @@ class CdoTest(testClass):
         ofile1 = cdo.setname("veloc").copy.random("r1x1").add_option("-f nc").run()
         cdo = Cdo()
         ofile2 = cdo.setname("veloc", input=" -copy -random,r1x1",options = "-f nc")
-        diff = cdo.diff(input=[ofile1, ofile2])
+        diff = cdo.diff(input=[ofile1, ofile2], options='-s')
         self.assertFalse(diff, msg=diff)
 
     def test_diff(self):
         cdo = Cdo()
         cdo.debug = DEBUG
         diffv = cdo.diffn(input = "-random,global_0.5 -random,global_0.5")
-        self.assertEqual('random', diffv[-2].split(' ')[-1],"random")
-        self.assertEqual('1 of 1 records differ',diffv[-1])
+        thisVersion = parse_version(cdo.version())
+        if (thisVersion >= parse_version('2.0.0') and thisVersion <= parse_version('2.0.5')):
+          self.assertEqual('random', diffv[-5].split(' ')[-1],"random")
+          self.assertEqual('1 of 1 records differ',diffv[-4])
+        else:
+          self.assertEqual('random', diffv[-2].split(' ')[-1],"random")
+          self.assertEqual('1 of 1 records differ',diffv[-1])
 
     def test_returnCdf(self):
         cdo = Cdo()
@@ -449,6 +460,15 @@ class CdoTest(testClass):
 
       xarrayFile.close()
 
+    def test_xarray_input_and_output(self):
+      cdo = Cdo()
+      ifile = cdo.setcalendar('360_day', input=' -settaxis,1800-01-01,12:00:00,1months -seq,1,10000',options='-f nc -r')
+      (var,) = cdo.showname(input=ifile)
+      self.assertEqual('seq',var)
+      x = cdo.selyear('1985/2100', input=ifile, returnXArray=var)
+      y = cdo.yearsum(input=x, returnXArray=var)
+      self.assertEqual(4059768.0,float(y[:,0,0].sum()))
+
     def test_xarray_output(self):
       cdo = Cdo()
       try:
@@ -491,16 +511,17 @@ class CdoTest(testClass):
     def test_inputArray(self):
         cdo = Cdo()
         cdo.debug = DEBUG
+        cdo.silent = True
         # check for file input
         fileA = cdo.stdatm(0,output='A_{0}'.format( random.randrange(1,100000)))
         fileB = cdo.stdatm(0,output='B_{0}'.format( random.randrange(1,100000)))
         files = [fileA,fileB]
-        self.assertEqual(cdo.diffv(input = ' '.join(files)), cdo.diffv(input = files))
-        self.assertEqual([],cdo.diffv(input = files))
+        self.assertEqual(cdo.diffv(input = ' '.join(files), options='-s'), cdo.diffv(input = files, options='-s'))
+        self.assertEqual([],cdo.diffv(input = files, options='-s'))
         # check for operator input
-        self.assertEqual([],cdo.diffv(input = ["-stdatm,0","-stdatm,0"]))
+        self.assertEqual([],cdo.diffv(input = ["-stdatm,0","-stdatm,0"], options='-s'))
         # check for operator input and files
-        self.assertEqual([],cdo.diffv(input = ["-stdatm,0",fileB]))
+        self.assertEqual([],cdo.diffv(input = ["-stdatm,0",fileB], options='-s'))
         rm([fileA, fileB])
 
     def test_splitOps(self):
@@ -796,6 +817,8 @@ class CdoTest(testClass):
       output = cdo.verifygrid(input='-topo,global_10')
       self.assertEqual([],output)
 
+      self.assertEqual(True,cdo.silent)
+
       cdo.silent = False
       output = cdo.verifygrid(input='-topo,global_10')
       if parse_version('2.0.6') <= parse_version(cdo.version()):
@@ -813,6 +836,8 @@ class CdoTest(testClass):
                           'cdo    verifygrid:        lat : -85 to 85 degrees',
                           'cdo(1) topo:',
                           'cdo    verifygrid: Processed 1 variable [0.00s 94MB].']
+        expectedOutput = expectedOutput[0:-2]
+        output = output[0:-2]
       elif parse_version('2.0.0') > parse_version(cdo.version()):
         # versions 1.9.x and earlier write to stderr and will not be tested
         expectedOutput = []
@@ -827,14 +852,24 @@ class CdoTest(testClass):
 
       def test_config(self):
         cdo = Cdo()
-        if (parse_version(cdo.version()) > parse_version('1.9.8')):
+        if (parse_version(cdo.version()) > parse_version('1.9.8') and
+            parse_version(cdo.version()) <= parse_version('2.0.0')):
           self.assertEqual({'has-cgribex': 'yes', 'has-cmor': 'no', 'has-ext': 'yes', 'has-grb': 'yes', 'has-grb1': 'yes', 'has-grb2': 'yes', 'has-hdf5': 'yes', 'has-ieg': 'yes', 'has-nc': 'yes', 'has-nc2': 'yes', 'has-nc4': 'yes', 'has-nc4c': 'yes', 'has-nc5': 'yes', 'has-openmp': 'yes', 'has-proj': 'yes', 'has-srv': 'yes', 'has-threads': 'yes', 'has-wordexp': 'yes'},
               cdo.config)
+        else:
+          self.assertEqual({'has-cgribex': 'yes', 'has-cmor': 'no', 'has-ext': 'yes', 'has-grb': 'yes', 'has-grb1': 'yes', 'has-grb2': 'yes', 'has-hdf5': 'yes', 'has-ieg': 'yes', 'has-nc': 'yes', 'has-nc2': 'yes', 'has-nc4': 'yes', 'has-nc4c': 'yes', 'has-nc5': 'yes', 'has-openmp': 'yes', 'has-proj': 'yes', 'has-srv': 'yes', 'has-threads': 'yes', 'has-wordexp': 'yes', 'has-nczarr': 'yes', 'has-magics': 'yes', 'has-hirlam_extensions': 'no'},
+              cdo.config)
 
       def test_system_tempdir(self):
-        # automatic path
-        tempPath = tempfile.gettempdir()
-        cdo = Cdo()
+        # system tempdir migh be cluttered with other stuff, so lets use a
+        # private directory
+        import tempfile
+        tempfile.tempdir = os.path.abspath('.')
+
+        # create and use local tempdir
+        tempPath = tempfile.mkdtemp()
+        cdo = Cdo(tempdir=tempPath)
+
         cdo.topo('r10x10',options = '-f nc')
         self.assertEqual(1,len([ f for f in os.listdir(tempPath) if 'cdoPy' in f]))
         cdo.topo('r10x10',options = '-f nc')
@@ -901,26 +936,32 @@ class CdoTest(testClass):
             self.assertEqual(sorted(['lat','lon','seq','time']),sorted(list(cdf.variables.keys())))
 
       def test_phc(self):
-        if os.path.isfile(DATA_DIR + '/icon/phc.nc'):
-          ifile = "-select,level=0 " + DATA_DIR + '/icon/phc.nc'
+        ifile = DATA_DIR + '/icon/phc.nc'
+        if os.path.isfile(ifile):
+          ifile = "-setctomiss,nan -select,level=0,timestep=1 " + DATA_DIR + '/icon/phc.nc'
           cdo = Cdo()
           cdo.debug = DEBUG
           if not cdo.hasNetcdf:
             return
+          saltVarname, tempVarname = 'salt', 'temp'
           #cdo.merge(input='/home/ram/data/icon/input/phc3.0/PHC__3.0__TempO__1x1__annual.nc /home/ram/data/icon/input/phc3.0/PHC__3.0__SO__1x1__annual.nc',
           #          output=ifile,
           #          options='-O')
-          s = cdo.sellonlatbox(0,30,0,90, input="-chname,SO,s,TempO,t " + ifile,output='test_my_phc.nc',returnMaArray='s',options='-f nc')
-          plot(np.flipud(s[0,:,:]),ofile='org',title='original')
-          sfmo = cdo.sellonlatbox(0,30,0,90, input="-fillmiss -chname,SO,s,TempO,t " + ifile,returnMaArray='s',options='-f nc')
-          plot(np.flipud(sfmo[0,:,:]),ofile='fm',title='fillmiss')
-          sfm = cdo.sellonlatbox(0,30,0,90, input="-fillmiss2 -chname,SO,s,TempO,t " + ifile,returnMaArray='s',options='-f nc')
-          plot(np.flipud(sfm[0,:,:]),ofile='fm2',title='fillmiss2')
-          ssetmisstonn = cdo.sellonlatbox(0,30,0,90, input="-setmisstonn -chname,SO,s,TempO,t " + ifile,returnMaArray='s',options='-f nc')
-          plot(np.flipud(ssetmisstonn[0,:,:]),ofile='setmisstonn',title='setmisstonn')
+          s = cdo.sellonlatbox(0,30,0,90, input=ifile,output='test_my_phc.nc',returnMaArray=saltVarname,options='-f nc')[0,0,:,:]
+          plot(s,ofile='org',title='original')
+
+          sfmo = cdo.sellonlatbox(0,30,0,90, input="-fillmiss " + ifile,returnMaArray=saltVarname,options='-f nc')[0,0,:,:]
+          plot(sfmo,ofile='fm',title='fillmiss')
+
+          sfm = cdo.sellonlatbox(0,30,0,90, input="-fillmiss2 " + ifile,returnMaArray=saltVarname,options='-f nc')[0,0,:,:]
+          plot(sfm,ofile='fm2',title='fillmiss2')
+
+          ssetmisstonn = cdo.sellonlatbox(0,30,0,90, input="-setmisstonn " + ifile,returnMaArray=saltVarname,options='-f nc')[0,0,:,:]
+          plot(ssetmisstonn,ofile='setmisstonn',title='setmisstonn')
+
           if (parse_version(cdo.version()) >= parse_version('1.7.2')):
-            smooth = cdo.sellonlatbox(0,30,0,90, input="-smooth -chname,SO,s,TempO,t " + ifile,returnMaArray='s',options='-f nc')
-            plot(np.flipud(ssetmisstonn[0,:,:]),ofile='smooth',title='smooth')
+            smooth = cdo.sellonlatbox(0,30,0,90, input="-smooth " + ifile,returnMaArray=saltVarname,options='-f nc')[0,0,:,:]
+            plot(ssetmisstonn,ofile='smooth',title='smooth')
           #global plot
           #s_global = cdo.chname('SO,s,TempO,t',input=ifile,output='my_phc.nc',returnMaArray='s',options='-f nc')
           #plot(s_global[0,:,:],ofile='org_global',title='org_global')
@@ -930,34 +971,58 @@ class CdoTest(testClass):
           #plot(sfm_global[0,:,:],ofile='fm2_global',title='fm2_global')
           #ssetmisstonn_global = cdo.setmisstonn(input=" -chname,SO,s,TempO,t " + ifile,returnMaArray='s',options='-f nc')
           #plot(ssetmisstonn_global[0,:,:],ofile='setmisstonn_global',title='setmisstonn_global')
+        else:
+          print(f"Could not find data '{ifile}'")
 
       def test_smooth(self):
         if os.path.isfile(DATA_DIR + '/icon/phc.nc'):
+          saltVarname, tempVarname = 'salt', 'temp'
+          selectionOperator = 'sellonlatbox'
+          region = "0,30,30,90"
+          regionSelection = f"-{selectionOperator},{region}"
           cdo = Cdo()
           if (parse_version(cdo.version()) >= parse_version('1.7.2') and cdo.hasNetcdf):
-            ifile = "-select,level=0 " + DATA_DIR + '/icon/phc.nc'
+            ifile = "-setctomiss,nan -select,level=0,timestep=1 " + DATA_DIR + '/icon/phc.nc'
             cdo = Cdo()
             cdo.debug = DEBUG
             #cdo.merge(input='/home/ram/data/icon/input/phc3.0/PHC__3.0__TempO__1x1__annual.nc /home/ram/data/icon/input/phc3.0/PHC__3.0__SO__1x1__annual.nc',
             #          output=ifile,
             #          options='-O')
-            smooth = cdo.smooth(input=" -sellonlatbox,0,30,0,90 -chname,SO,s,TempO,t " + ifile, returnMaArray='s',options='-f nc')
-            plot(np.flipud(smooth[0,:,:]),ofile='smooth',title='smooth')
+            org, tag = cdo.sellonlatbox(region,input=ifile, returnMaArray=saltVarname)[0,0,:,:], 'org'
+            plot(org,ofile=tag,title=tag)
+            smooth, tag = cdo.smooth(input=f" {regionSelection} {ifile}",
+                                options='-L',
+                                returnMaArray=saltVarname)[0,0,:,:], 'smooth'
+            plot(smooth,ofile=tag,title=tag)
 
-            smooth2 = cdo.smooth('nsmooth=2',input="-sellonlatbox,0,30,0,90 -chname,SO,s,TempO,t " + ifile, returnMaArray='s',options='-f nc')
-            plot(np.flipud(smooth2[0,:,:]),ofile='smooth2',title='smooth,nsmooth=2')
+            smooth2, tag = cdo.smooth('nsmooth=2',
+                                 input=f" {regionSelection} {ifile}",
+                                 options='-L',
+                                 returnMaArray=saltVarname)[0,0,:,:], 'smooth2'
+            plot(np.flipud(smooth2),ofile=tag,title='smooth,nsmooth=2')
+            print(smooth - org)
 
-            smooth4 = cdo.smooth('nsmooth=4',input="-sellonlatbox,0,30,0,90 -chname,SO,s,TempO,t " + ifile, returnMaArray='s',options='-f nc')
-            plot(np.flipud(smooth4[0,:,:]),ofile='smooth4',title='smooth,nsmooth=4')
+            smooth4 = cdo.smooth('nsmooth=4',
+                                 input=f" {regionSelection} {ifile}",
+                                 options='-L',
+                                 returnMaArray=saltVarname)[0,0,:,:]
+            plot(np.flipud(smooth4),ofile='smooth4',title='smooth,nsmooth=4')
 
-            smooth9 = cdo.smooth9(input="-sellonlatbox,0,30,0,90 -chname,SO,s,TempO,t " + ifile, returnMaArray='s',options='-f nc')
-            plot(np.flipud(smooth9[0,:,:]),ofile='smooth9',title='smooth9')
+            smooth9 = cdo.smooth9(input=f" {regionSelection} {ifile}",
+                                  options='-L',
+                                  returnMaArray=saltVarname)[0,0,:,:]
+            plot(np.flipud(smooth9),ofile='smooth9',title='smooth9')
 
-            smooth3deg = cdo.smooth('radius=6deg',input="-sellonlatbox,0,30,0,90 -chname,SO,s,TempO,t " + ifile, returnMaArray='s',options='-f nc')
-            plot(np.flipud(smooth3deg[0,:,:]),ofile='smooth3deg',title='smooth,radius=6deg')
+            smooth3deg = cdo.smooth('radius=6deg',
+                                    input=f" {regionSelection} {ifile}",options='-L',
+                                    returnMaArray=saltVarname)[0,0,:,:]
+            plot(np.flipud(smooth3deg),ofile='smooth3deg',title='smooth,radius=6deg')
 
-            smooth20 = cdo.smooth('nsmooth=20',input="-sellonlatbox,0,30,0,90 -chname,SO,s,TempO,t " + ifile, returnMaArray='s',options='-f nc')
-            plot(np.flipud(smooth20[0,:,:]),ofile='smooth20',title='smooth,nsmooth=20')
+            smooth20 = cdo.smooth('nsmooth=20',
+                                  input=f" {regionSelection} {ifile}",
+                                  options='-L',
+                                  returnMaArray=saltVarname)[0,0,:,:]
+            plot(np.flipud(smooth20),ofile='smooth20',title='smooth,nsmooth=20')
 
       def test_ydiv(self):
         cdo = Cdo()
@@ -981,6 +1046,19 @@ class CdoTest(testClass):
             options = ' -r -f nc',
             input = "-setname,tmax -setctomiss,-999.99 -input,{} tmax.nc < ".format(gridfile),
             output = inputfile)
+
+      def test_pauline(self):
+        ifile='/home/ram/local/data/cdo/pauline.nc'
+        cdo = Cdo()
+        (var,) = cdo.showname(input=ifile)
+        x = cdo.selyear('1985/2100', input=ifile, returnXArray=var)
+        print(x)
+        self.assertEqual((4018, 7, 5),x[:,:,:].shape)
+        y = cdo.yearsum(input=x, returnXArray=var)
+        print(y)
+        self.assertEqual((11, 7, 5),y[:,:,:].shape)
+
+
 #===============================================================================
 if __name__ == '__main__':
     suite = testLoader.loadTestsFromTestCase(CdoTest)
